/*---------------------------------------------------------*\
| startup.h                                                 |
|                                                           |
|   Startup for the OpenRGB application                     |
|                                                           |
|   This file is part of the OpenRGB project                |
|   SPDX-License-Identifier: GPL-2.0-or-later               |
\*---------------------------------------------------------*/

#pragma once

int startup(int argc, char* argv[], unsigned int ret_flags);
